@@ -7,6 +7,7 @@
 package Graph::Easy;
 
 use 5.008000;
+use Graph::Easy::Base;
 use Graph::Easy::Attributes;
 use Graph::Easy::Edge;
 use Graph::Easy::Group;
@@ -17,7 +18,8 @@ use Graph::Easy::Node::Anon;
 use Graph::Easy::Node::Empty;
 use Scalar::Util qw/weaken/;
 
-$VERSION = '0.33';
+$VERSION = '0.34';
+@ISA = qw/Graph::Easy::Base/;
 
 use strict;
 
@@ -34,8 +36,10 @@ BEGIN
   }
 
 #############################################################################
+
 sub new
   {
+  # override new() as to not set the {id}
   my $class = shift;
 
   my $self = bless {}, $class;
@@ -50,6 +54,7 @@ sub DESTROY
   {
   my $self = shift;
 
+  delete $self->{chains};
   # clean out pointers in child-objects so that they can safely be reused
   for my $n (values %{$self->{nodes}})
     {
@@ -57,6 +62,9 @@ sub DESTROY
       {
       $n->{edges} = undef;
       $n->{graph} = undef;
+      delete $n->{_chain};
+      delete $n->{_c};
+      delete $n->{_next};
       }
     }
   for my $e (values %{$self->{edges}})
@@ -88,9 +96,6 @@ sub _init
   $self->{edges} = {};
 
   $self->{output_format} = 'html';
-  $self->{html_header} = '';
-  $self->{html_footer} = '';
-  $self->{html_style} = '';
 
   $self->{_astar_bias} = 0.001;
 
@@ -274,14 +279,6 @@ sub seed
   $self->{seed};
   }
 
-sub error
-  {
-  my $self = shift;
-
-  $self->{error} = $_[0] if defined $_[0];
-  $self->{error} || '';
-  }
-
 sub nodes
   {
   # return all nodes as objects
@@ -350,10 +347,8 @@ sub sorted_nodes
   $sort = sub { $a->{$f1} <=> $b->{$f1} || $a->{$f2} cmp $b->{$f2} } if $f2 &&
            $f2 =~ /^(name|title|label)$/;
 
-  my @nodes = values %{$self->{nodes}};
-
   # the 'return' here should not be removed
-  return sort $sort @nodes;
+  return sort $sort values %{$self->{nodes}};
   }
 
 sub edge
@@ -720,6 +715,13 @@ table.graph##id## td {
 CSS
 ;
 
+# eb and eh seem not longer nec.:
+# table.graph##id## .eb {
+#  height: 1em;
+#  max-height: 1em;
+#  line-height: 0.5em;
+#  }
+
   # count anon nodes and append CSS for them if nec.
   $css .= <<CSSANON
 table.graph##id## .node-anon {
@@ -732,26 +734,23 @@ CSSANON
   # (left/right), vertical empty), etc)
   $css .= <<CSS
 table.graph##id## .va {
-  vertical-align: bottom;
+  vertical-align: center;
   line-height: 1.5em;
   width: 0.4em;
   }
 table.graph##id## .ve {
   width: 0em;
   }
+table.graph##id## .el {
+  width: 1em;
+  max-width: 1em;
+  }
 table.graph##id## .lh, table.graph##id## .lv {
   font-size: 0.8em;
   padding-left: 0.4em;
   }
-table.graph##id## .eb {
-  height: 1em;
-  line-height: 0.5em;
-  }
-table.graph##id## .el {
-  width: 0.5em;
-  }
-table.graph##id## .v, table.graph##id## .hat, table.graph##id## .he {
-  text-align: left;
+table.graph##id## .v, table.graph##id## .hat {
+  text-align: center;
   height: 0.5em;
   line-height: 0.6em;
   }
@@ -851,7 +850,7 @@ sub as_html
 
   $self->layout() unless defined $self->{score};
 
-  my $top = "\n" . $self->{html_header};
+  my $top = "\n";
 
   my $cells = $self->{cells};
   my ($rows,$cols);
@@ -884,7 +883,6 @@ sub as_html
   my $id = $self->{id};
 
   $top .=  "\n<table class=\"graph$id\" cellpadding=0 cellspacing=0";
-  $top .= " style=\"$self->{html_style}\"" if $self->{html_style};
   $top .= ">\n";
 
   my $html = '';
@@ -1027,7 +1025,7 @@ sub as_html
   # remove empty trailing <tr></tr> pairs
   $html =~ s#(<tr></tr>\n\n)+\z##;
 
-  $html .= "</table>\n" . $self->{html_footer};
+  $html .= "</table>\n";
  
   $html;
   } 
@@ -1089,6 +1087,7 @@ sub _as_ascii
   my $self = shift;
 
   require Graph::Easy::As_ascii;
+  require Graph::Easy::Layout::Grid;
 
   my $opt = ref($_[0]) eq 'HASH' ? $_[0] : { @_ };
 
@@ -1098,6 +1097,7 @@ sub _as_ascii
   $self->layout() unless defined $self->{score};
 
   # generate for each cell the width/height etc
+
   my ($rows,$cols,$max_x,$max_y,$cells) = $self->_prepare_layout('ascii');
 
   my $y_start = 0;
@@ -1179,159 +1179,6 @@ sub as_ascii_html
   "<pre>\n" . $self->_as_ascii(@_) . "\n</pre>\n";
   }
 
-#############################################################################
-
-sub _prepare_layout
-  {
-  # this method is used by as_ascii() and as_svg() to find out the
-  # sizes and placement of the different cells (edges, nodes etc).
-  my ($self,$format) = @_;
-
-  # Find out for each row and colum how big they are:
-  #   +--------+-----+------+
-  #   | Berlin | --> | Bonn | 
-  #   +--------+-----+------+
-  # results in:
-  #        w,  h,  x,  y
-  # 0,0 => 10, 3,  0,  0
-  # 1,0 => 7,  3,  10, 0
-  # 2,0 => 8,  3,  16, 0
-
-  # Technically, we also need to "compress" away non-existant columns/rows.
-  # We achive that by simply rendering them with size 0, so they become
-  # practically invisible.
-
-  my $cells = $self->{cells};
-  my $rows = {};
-  my $cols = {};
-  my @V;
-
-  # the last column/row
-  my $mx = -100000; my $my = -100000;
-
-  # find all x and y occurances to sort them by row/columns
-  for my $k (keys %$cells)
-    {
-    my ($x,$y) = split/,/, $k;
-    my $cell = $cells->{$k};
-
-    # Get all possible nodes from $cell (instead of nodes) because
-    # this also includes edge/group cells, too.
-    push @V, $cell;
-
-    # Set the minimum cell size:
-    {
-      no strict 'refs';
-
-      my $method = '_correct_size_' . $format;
-      $method = '_correct_size' unless $cell->can($method);
-      $cell->$method();
-    }
-
-    my $w = $cell->{w};
-    my $h = $cell->{h};
-
-    # record maximum size for that col/row
-    $rows->{$y} = $h if $h >= ($rows->{$y} || 0);
-    $cols->{$x} = $w if $w >= ($cols->{$x} || 0);
-
-    $mx = $x if $x > $mx;
-    $my = $y if $y > $my;
-    } 
-
-  # insert a dummy row/column with size=0 as last
-  $rows->{$my+1} = 0;
-  $cols->{$mx+1} = 0;
-
-  # Now run through all rows/columns and get their absolute pos by taking all
-  # previous ones into account.
-  my $pos = 0;
-  for my $y (sort { $a <=> $b } keys %$rows)
-    {
-    my $s = $rows->{$y};
-    $rows->{$y} = $pos;			# first is 0, second is $rows[1] etc
-    $pos += $s;
-    }
-  $pos = 0;
-  for my $x (sort { $a <=> $b } keys %$cols)
-    {
-    my $s = $cols->{$x};
-    $cols->{$x} = $pos;
-    $pos += $s;
-    }
-
-  # find out max. dimensions for framebuffer
-  my $max_y = 0; my $max_x = 0;
-
-  foreach my $v (@V)
-    {
-    next if ($v->{cx}||1) + ($v->{cy}||1) != 2;
-
-    # X and Y are col/row, so translate them to real pos
-    my $x = $cols->{ $v->{x} };
-    my $y = $rows->{ $v->{y} };
-
-    # Also set correct the width/height of each cell to be the maximum
-    # width/height of that colum and store the previous size in 'minw'
-    # and 'minh', respectively. Skip multi-celled nodes for later. 
-
-    $v->{minw} = $v->{w};
-    $v->{minh} = $v->{h};
-
-    # find next col/row
-    my $nx = $v->{x} + 1;
-    my $next_col = $cols->{ $nx };
-    my $ny = $v->{y} + 1;
-    my $next_row = $rows->{ $ny };
-
-    $next_col = $cols->{ ++$nx } while (!defined $next_col);
-    $next_row = $rows->{ ++$ny } while (!defined $next_row);
-
-    $v->{w} = $next_col - $x;
-    $v->{h} = $next_row - $y;
-
-    my $m = $y + $v->{h} - 1;
-    $max_y = $m if $m > $max_y;
-    $m = $x + $v->{w} - 1;
-    $max_x = $m if $m > $max_x;
-    }
-
-  # repeat the previous step, now for multi-celled nodes
-  # Looping over nodes should be faster
-  foreach my $v (values %{$self->{nodes}})
-    {
-    next unless defined $v->{x} && (($v->{cx}||1) + ($v->{cy}||1) > 2);
-
-    # X and Y are col/row, so translate them to real pos
-    my $x = $cols->{ $v->{x} };
-    my $y = $rows->{ $v->{y} };
-
-    $v->{minw} = $v->{w};
-    $v->{minh} = $v->{h};
-
-    # find next col/row
-    my $nx = $v->{x} + $v->{cx};
-    my $next_col = $cols->{ $nx };
-    my $ny = $v->{y} + $v->{cy};
-    my $next_row = $rows->{ $ny };
-
-    $next_col = $cols->{ ++$nx } while (!defined $next_col);
-    $next_row = $rows->{ ++$ny } while (!defined $next_row);
-
-    $v->{w} = $next_col - $x;
-    $v->{h} = $next_row - $y;
-
-    my $m = $y + $v->{h} - 1;
-    $max_y = $m if $m > $max_y;
-    $m = $x + $v->{w} - 1;
-    $max_x = $m if $m > $max_x;
-    }
-
-  # return what we found out:
-
-  ($rows,$cols,$max_x,$max_y, \@V);
-  }
-
 #############################################################################
 # as_txt, as_graphviz and as_svg
 
@@ -1345,6 +1192,7 @@ sub as_graphviz
 sub as_svg
   {
   require Graph::Easy::As_svg;
+  require Graph::Easy::Layout::Grid;
 
   _as_svg(@_);
   }
@@ -1352,10 +1200,24 @@ sub as_svg
 sub as_svg_file
   {
   require Graph::Easy::As_svg;
+  require Graph::Easy::Layout::Grid;
 
   _as_svg( $_[0], { standalone => 1 } );
   }
 
+sub svg_information
+  {
+  my ($self) = @_;
+
+  require Graph::Easy::As_svg;
+  require Graph::Easy::Layout::Grid;
+
+  # if it doesn't exist, render as SVG and thus create it
+  _as_svg(@_) unless $self->{svg_info};
+
+  $self->{svg_info};
+  }
+
 sub as_txt
   {
   require Graph::Easy::As_txt;
@@ -2287,6 +2149,24 @@ See also C<as_svg_file()>.
 Returns SVG just like C<as_svg()>, but this time as standalone SVG,
 suitable for storing it in a file and referencing it externally.
 
+After calling C<as_svg_file()> or C<as_svg()>, you can retrieve
+some SVG information, notable C<width> and C<height> via
+C<svg_information>.
+
+=head2 svg_information()
+
+	my $info = $graph->svg_information();
+
+	print "Size: $info->{width}, $info->{height}\n";
+
+Return information about the graph created by the last
+C<as_svg()> or C<as_svg_file()> call.
+
+The following fields are set:
+
+	width		width of the SVG in pixels
+	height		height of the SVG in pixels
+
 =head2 nodes()
 
 	my $nodes = $graph->nodes();
